# * Imports a Magix VPX Edl into the Vegas Timeline
# *
# * Revision Date: Jan 15,2019
# * Version:       1.0
# * Copyright:     Harold Linke 
# 

clr.AddReference('System.Windows.Forms')
import System.Windows.Forms as WinForms
import os

# List directories where missing media should be searched
searchDirList = ["F:\\data_archiv\\Videos1\\Original Videos", "E:\data_archiv\Video_Projekte\_all_video_projects", "E:\\MMserver_NAS3\\Musik"]
searchMissingMedia = False # set to TRUE if you want that VEGAS is looking for missing media


searchStringFilenameListStart = "Source Table Entries:"
searchStringFilenameListEnd = "Track"
searchStringTrackListStart = "Track"
searchStringTrackListEnd = "#Vol"


class EDLFile:
    Filename = ""
    File = None
    fileIsOpen = False
    lastLineRead = ""
    filenamelist = [[0,"",0]] # list of filename items [id,filename,type]
    trackListEnd = False
    trackEventList = [[]]
    timeFactor = 48
    
    def __init__(self):
        self.trackListEnd = False

    def openFile (self,newfilename):
        """ open the edl file

        Parameters:
            newfilename - name of the file to open

        return:
            True:  File is open
            False: File could not be opened

        Post condition:
            text file with name newfilename is open for reading
        """
        self.File = open(newfilename)
        if (self.File != None):
            self.fileIsOpen = True
            self.filename = newfilename
        else:
            self.fileIsOpen = False
        return self.fileIsOpen

    def closeFile(self):
        """ close the edl file

        Parameters:
            none

        return:
            none

        Pre condition
            file was open for reading

        Post condition:
            text file with name newfilename is closed
        """
        self.File.close()

    def readNextNELine(self):
        """ read next not empty line from edl file

        Parameters:
            none

        return: 
            String: next not empty line from edl file

        Pre condition
            file was open for reading

        Post condition:
            file read pointer points to begin of line after the read line
        """
        line = self.File.readline()
        if line:
            while line =="" or line == "\n":
                line = self.File.readline()
                if line == None:
                    line = ""
                    break
        return line

    def findStartEventFilenameList(self):
        """ find the start of the event filename list in the edl file

        Parameters:
            none

        return: 
            Bool: True - start of the event filename list in the edl file found

        Pre condition
            file was open for reading

        Post condition:
            file read pointer points to begin of event filename list
        """
        line=self.readNextNELine()
        self.lastLineRead = line
        while line !="":
            if searchStringFilenameListStart in line:
                return True
            line=self.readNextNELine()
        return False

    def findNextEventFilename(self):
        """ find the next event filename in the edl file

        Parameters:
            none

        return: 
            Bool: True - event filename list continues
                  False - end of event file name list found

        Pre condition
            file was open for reading
            file read pointer points to lines in event file list (findStartEventFilenameList was executed before)

        Post condition:
            file read pointer points to end of event filename list and the begin of the Track list
            lastLineRead contains the last line read - needed at the end of the list, as this line starts the "Track" list
        """
        line=self.readNextNELine()
        self.lastLineRead = line
        if line != "":
            if line.find(searchStringFilenameListEnd) == 0: # "Track" at start of line
                return False
            else:
                itemstrings = line.split('"') # 1: id 2: filename 3:eventtype (Video,Audio)
                id = int(itemstrings[0])
                filenameelem=[id,itemstrings[1],itemstrings[2]]
                self.filenamelist.append(filenameelem)
        return True

    def createEventFilenameList(self):
        """ create the event filename list from the edl file

        Parameters:
            none

        return: 
            none

        Pre condition
            file was open for reading
            file read pointer points to lines before the event file list

        Post condition:
            file read pointer points to end of event filename list and the begin of the Track list
            lastLineRead contains the last line read - needed at the end of the list, as this line starts the "Track event list
            filenamelist contains the complete list of all mediafiles used in the edl file
        """
        nextLine = self.findStartEventFilenameList()
        while nextLine:
             nextLine = self.findNextEventFilename()
        return
    
    def findStartTrackList(self):
        """ find the start of a track list in the edl file

        Parameters:
            none

        return: 
            Bool: True - track list start found
                  False - no tracklist found anymore

        Pre condition
            file was open for reading
            file read pointer points after the first line in the track event list with the "Track" keyword
            lastLineRead contains the last line read with the "Track" keyword

        Post condition:
            file read pointer points to first event in the event list for the track
            lastLineRead contains the last line read
            
        """
        if self.lastLineRead.find(searchStringFilenameListEnd) == 0: # "Track" at start of line
            line=self.readNextNELine()
            self.lastLineRead = line
            #skip the source and the header lines
            if line.find("#Source") == 0:
                line=self.readNextNELine()
                self.lastLineRead = line
                if line.find("#------") == 0:
                    return True
        return False

    def findNextTrackEvent(self):
        """ find the next event in the track list in the edl file

        Parameters:
            none

        return: 
            Bool: True - next event found
                  False - end of event list for this track reached

        Pre condition
            file was open for reading
            file read pointer points on the first line in the track event list
            lastLineRead contains the last line read

        Post condition:
            file read pointer points to next event in the event list for the track
            lastLineRead contains the last line read
            the event properties read from the edl file are appended to the trackEventList
            
        """
        line=self.readNextNELine()
        self.lastLineRead = line
        if line != "":
            if line.find(searchStringTrackListStart) == 0 or line.find(searchStringTrackListEnd) == 0: # # "Track" at start of line "#Volume" at start of line - end of tracklist
                return False
            else:
                itemstrings = line.split() # 0: id 1:track 2:Play-In 3: Play-Out 4:Record-In 5:Record-Out 6:Vol 7:MT 8:LK 9:FadeIn 10:% 11:CurveType 12:FadeOut 13:% 14:CurveType 15:Name
                fileProperties = self.filenamelist[int(itemstrings[0])]
                trackType = "Video"
                if "AUDIO" in fileProperties[2]:
                    trackType = "Audio"
 
                #                  0:Audio/Video  1:pathfilename    2:Track number      3:PlayIn                            4:PlayOut                           5:RecordIn                          6:RecordOut                         7:FadeIn                            8:FadeOut
                eventProperties = [trackType,     fileProperties[1],int(itemstrings[1]),int(itemstrings[2])/self.timeFactor,int(itemstrings[3])/self.timeFactor,int(itemstrings[4])/self.timeFactor,int(itemstrings[5])/self.timeFactor,int(itemstrings[9])/self.timeFactor,int(itemstrings[12])/self.timeFactor]
                self.trackEventList.append(eventProperties)
                return True
        else:
            return False

    def getTrackEventList(self):
        """ get the track event list from the edl file

        Parameters:
            none

        return: 
            trackEventList with all events and all tracks defined in the edl file

        Pre condition
            file was open for reading
            file read pointer points the start of the edl file
 
        Post condition:
            file read pointer points to theend event list for the last track
            lastLineRead contains the last line read
            all event properties read from the edl file are appended to the trackEventList for all tracks
            
        """
        trackEventList = []
        while self.findStartTrackList():
            while self.findNextTrackEvent():
                pass
        return self.trackEventList

    #*** Class EDLFile end *********


def checkFileExists(filePathName):
    """ check if file exists

    Parameters:
        filePathName: Path and name of the file to check

        when the file was not found and searchMissingMedia = True the procedure looks in all subdirectories of the directories specified in searchDirList

    return: 
        original filePathName or updated filePathName with new path
        "" when file was not found

    Pre condition
        none

    Post condition:
        none
            
    """
    fname = os.path.basename(filePathName)
    if os.path.isfile(filePathName):
        return filePathName
    elif searchMissingMedia:
        for rootDir in searchDirList:
            print "Search for ", fname, " in ", rootDir
            for dirName,subdirList, fileList in os.walk(rootDir):
                if fname in fileList:
                    return os.path.join(dirName, fname)
    print (fname, " not found")
    return ""

  
def addVideoEvent(eventProp,Track):
    """ add a video event to the track

    Parameters:
        eventProp: list of event properties
                   0:Audio/Video
                   1:pathfilename
                   2:Track number
                   3:PlayIn (in ms)
                   4:PlayOut (in ms)
                   5:RecordIn (in ms)
                   6:RecordOut (in ms)
                   7:FadeIn (in ms)
                   8:FadeOut (in ms)
        track:     track, to which the event should be added

    return: 
        none

    Pre condition
        track exists in VEGAS timeline

    Post condition:
        new video event with properties defined in eventProp is added to the track in the timeline
        if the mediafile is not existing dummy media is create
            
    """

    mediafilename = eventProp[1] 
    tlStart = Timecode.FromMilliseconds(eventProp[3]) # PlayIn
    takeStart = Timecode.FromMilliseconds(eventProp[5]) # RecordIn
    eventLength = Timecode.FromMilliseconds(eventProp[4]) - tlStart #PlayOut-PlayIn
    FadeIn = Timecode.FromMilliseconds(eventProp[7]) # FadeIn
    FadeOut = Timecode.FromMilliseconds(eventProp[8]) # FadeOut
    curveType = CurveType.Slow # Standard Curve type for FadeIn, Fadeout
    
    checkedmediafilename = checkFileExists(mediafilename)
    if checkedmediafilename == "":
        media = Media.CreateInstance(pyVEGAS.Project,mediafilename)
        mediastream = Media.CreateOfflineStream(media,MediaType.Video)
        maxlength = eventLength
    else:
        media = Media.CreateInstance(pyVEGAS.Project,checkedmediafilename)
        mediastream = media.GetVideoStreamByIndex(0)
        maxlength = media.Length

    Event = Track.AddVideoEvent(tlStart,maxlength)
    take = Event.AddTake(mediastream)
    dStart = Event.Start + takeStart
    Event.AdjustStartLength(dStart, eventLength, True)
    Event.Start = tlStart

    Event.FadeIn.Curve=curveType
    Event.FadeOut.Curve=curveType
    Event.FadeIn.Length = FadeIn
    Event.FadeOut.Length = FadeOut

def addAudioEvent(eventProp,Track):
    """ add an audio event to the track

    Parameters:
        eventProp: list of event properties
                   0:Audio/Video
                   1:pathfilename
                   2:Track number
                   3:PlayIn (in ms)
                   4:PlayOut (in ms)
                   5:RecordIn (in ms)
                   6:RecordOut (in ms)
                   7:FadeIn (in ms)
                   8:FadeOut (in ms)
        track:     track, to which the event should be added

    return: 
        none

    Pre condition
        track exists in VEGAS timeline

    Post condition:
        new audio event with properties defined in eventProp is added to the track in the timeline
        if the mediafile is not existing dummy media is create
            
    """
    mediafilename = eventProp[1] 
    tlStart = Timecode.FromMilliseconds(eventProp[3]) # Play-In
    takeStart = Timecode.FromMilliseconds(eventProp[5])
    eventLength = Timecode.FromMilliseconds(eventProp[4]) - tlStart
    FadeIn = Timecode.FromMilliseconds(eventProp[7]) # FadeIn
    FadeOut = Timecode.FromMilliseconds(eventProp[8]) # FadeOut
    curveType = CurveType.Slow # Standard Curve type for FadeIn, Fadeout
  
    
    checkedmediafilename = checkFileExists(mediafilename)
    if checkedmediafilename == "":
        media = Media.CreateInstance(pyVEGAS.Project,mediafilename)
        mediastream = Media.CreateOfflineStream(media,MediaType.Audio)
        maxlength = eventLength
    else:
        media = Media.CreateInstance(pyVEGAS.Project,checkedmediafilename)
        mediastream = media.GetAudioStreamByIndex(0)
        maxlength = media.Length

    Event = Track.AddAudioEvent(tlStart,maxlength)
    take = Event.AddTake(mediastream)
    dStart = Event.Start + takeStart
    Event.AdjustStartLength(dStart, eventLength, True)
    Event.Start = tlStart

    Event.FadeIn.Curve=curveType
    Event.FadeOut.Curve=curveType
    Event.FadeIn.Length = FadeIn
    Event.FadeOut.Length = FadeOut



def createTimeline(trackEventList):
    """ creates the timeline from the trackEventList entries

    Parameters:
        trackEventList
        
    return: 
        none

    Pre condition
        trackEventList contains all events from the edl file

    Post condition:
        all tracks and events defined in the edl file are added to the timeline
            
    """
    lastTrackNo = 0

    for eventFileProp in trackEventList:
        if eventFileProp != []:
            curTrackNo = eventFileProp[2]
        
            if curTrackNo != lastTrackNo:
                # new track
                print ("Track No:", curTrackNo)
                lastTrackNo = curTrackNo
                if eventFileProp[0] == "Audio":
                    audioTrack = pyVEGAS.Project.AddAudioTrack()
                    audioTrack.Name = "Track " + str(curTrackNo)
                else:
                    videoTrack = pyVEGAS.Project.AddVideoTrack()
                    videoTrack.Name = "Track " + str(curTrackNo)
                
            if eventFileProp[0] == "Audio":
                addAudioEvent(eventFileProp,audioTrack)
            else:
                addVideoEvent(eventFileProp,videoTrack)


def FromVegas (myVegas):
    edlfile = EDLFile()

    openFileDialog1 = WinForms.OpenFileDialog();
    openFileDialog1.Filter = "Magix VPX EDL(*.edl)|*.edl|All files (*.*)|*.*";
    openFileDialog1.FilterIndex = 1;
    openFileDialog1.RestoreDirectory = True;
    openFileDialog1.AddExtension = True;
#    openFileDialog1.InitialDirectory = "\\VEGASPython";
    openFileDialog1.Title = "Load Magix VPX EDL";

    if (openFileDialog1.ShowDialog() == WinForms.DialogResult.OK):
        if edlfile.openFile(openFileDialog1.FileName):
            print("Create Filename list")
            edlfile.createEventFilenameList()
            print("Filename list created")
            print("Create Track Event list")
            trackEventList = edlfile.getTrackEventList()
            edlfile.closeFile()
            print("Track Event list created")
            print("Create Timeline")
            createTimeline(trackEventList)
            print("Timeline created")

FromVegas(pyVEGAS)

    
    
    
